/*
 * Copyright (C) 2006  Chris Rose
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package com.aimedia.azjabber;

import java.util.Date;

import org.gudy.azureus2.plugins.PluginConfig;
import org.gudy.azureus2.plugins.PluginException;
import org.gudy.azureus2.plugins.PluginInterface;
import org.gudy.azureus2.plugins.PluginListener;
import org.gudy.azureus2.plugins.UnloadablePlugin;
import org.gudy.azureus2.plugins.logging.LoggerChannel;
import org.gudy.azureus2.plugins.logging.LoggerChannelListener;
import org.gudy.azureus2.plugins.ui.model.BasicPluginViewModel;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;

import com.aimedia.azjabber.config.IAzJabberMessageKeys;
import com.aimedia.azjabber.config.IJabberConfigurationChangeListener;
import com.aimedia.azjabber.config.JabberConfiguration;
import com.aimedia.azjabber.config.JabberConnectionParameters;
import com.aimedia.azjabber.xmpp.XMPPConnectionManager;

public class AzJabberPlugin implements UnloadablePlugin {

    private static AzJabberPlugin instance     = null;

    public static AzJabberPlugin getPlugin () {
        return instance;
    }

    private PluginInterface     pluginInterface;

    /**
     * The plugin's name. This <strong>MUST MATCH THE DIRECTORY AND JAR NAMES</strong>
     */
    public static final String  PLUGIN_NAME = "azjabber";

    private JabberConfiguration configuration;

    private LoggerChannel logger;

    /*
     * (non-Javadoc)
     * 
     * @see org.gudy.azureus2.plugins.Plugin#initialize(org.gudy.azureus2.plugins.PluginInterface)
     */
    public void initialize (final PluginInterface pluginInterface) throws PluginException {
        instance = this;
        synchronized (instance) {
            this.pluginInterface = pluginInterface;

            final BasicPluginViewModel view = configurePluginView (pluginInterface);

            logger = pluginInterface.getLogger ().getTimeStampedChannel (PLUGIN_NAME);
            configureLogger (logger, view);

            configureJabberConfiguration ();

            NotificationManager.instance ().allServicesConfigured ();

            // : initialize the notification manager
            this.pluginInterface.getDownloadManager ().addListener (NotificationManager.instance ());

            // : initialize the XMPP connection manager
            XMPPConnectionManager.instance ().addServiceConnectionEventListener (NotificationManager.instance ());

            connectPluginInterface ();
        }
    }

    private void configureJabberConfiguration () {
        configuration = new JabberConfiguration ();

        configuration.addJabberConfigurationChangeListener (new IJabberConfigurationChangeListener () {
            public void jabberConfigurationChanged (JabberConnectionParameters parameters) {
                try {
                    NotificationManager.instance ().setRecipientPrefix (
                            parameters.getUsername () + "@" + parameters.getService ());
                    XMPPConnectionManager.instance ().connect (parameters);
                }
                catch (Exception e) {
                    Logger.error (IAzJabberMessageKeys.LOG_PLUGIN_CONNECT_FAILED, e);
                }
            }
        });
        
    }

    private void configureLogger (LoggerChannel log, final BasicPluginViewModel view) {
        log.addListener (new LoggerChannelListener () {

            public void messageLogged (int type, String content) {
                view.getLogArea ().appendText ("[" + new Date ().toString () + "] " + content + "\n");
            }

            public void messageLogged (String str, Throwable error) {
                view.getLogArea ().appendText ("[" + new Date ().toString () + "] " + str + "\n");
                if (error != null) {
                    view.getLogArea ().appendText (error.getLocalizedMessage () + "\n");
                }
            }
        });
    }

    private BasicPluginViewModel configurePluginView (final PluginInterface pluginInterface) {
        final BasicPluginViewModel view = pluginInterface.getUIManager ().createBasicPluginViewModel (
                pluginInterface.getUtilities ().getLocaleUtilities ().getLocalisedMessageText (
                        IAzJabberMessageKeys.PLUGIN_NAME));
        view.getActivity ().setVisible (false);
        view.getProgress ().setVisible (false);
        view.getStatus ().setVisible (false);
        view.setConfigSectionID( AzJabberPlugin.PLUGIN_NAME);
        return view;
    }

    private void connectPluginInterface () {
        // : Listen to the plugin events to shut down gracefully.
        this.pluginInterface.addListener (new PluginListener () {

            public void closedownComplete () {
                shutDown ();
            }

            public void closedownInitiated () {
                // : NO-OP
            }

            public void initializationComplete () {
                startUp ();
            }

        });
    }

    /**
     * @return Returns the pluginInterface.
     */
    public final PluginInterface getPluginInterface () {
        return pluginInterface;
    }

    public final LocaleUtilities getLocale () {
        return pluginInterface.getUtilities ().getLocaleUtilities ();
    }

    public final JabberConfiguration getConfiguration () {
        return this.configuration;
    }

    public void unload () throws PluginException {
        Logger.info (IAzJabberMessageKeys.LOG_SHUTDOWN);
    }

    private void startUp () {
        // : ensure that the initialize method is done.
        synchronized (instance) {
            Logger.info (IAzJabberMessageKeys.LOG_STARUPT);

            if (!configuration.isConfigured ()) {
                Logger.info (IAzJabberMessageKeys.LOG_NOT_CONFIGURED);
                return;
            }

            // : associate the notification manager with a service name.
            NotificationManager.instance ().setRecipientPrefix (
                    configuration.getClientUsername () + "@" + configuration.getJabberService ());

            try {
                XMPPConnectionManager.instance ().connect (getConfiguration ().getJabberConnectionParameters ());
            }
            catch (Exception e) {
                Logger.alert (IAzJabberMessageKeys.LOG_PLUGIN_CONNECT_FAILED, e);
            }
        }
    }

    private void shutDown () {
        Logger.info (IAzJabberMessageKeys.LOG_SHUTDOWN);

        if (XMPPConnectionManager.instance ().isConnected ()) {
            XMPPConnectionManager.instance ().disconnect ();
        }
    }
    
    public static final PluginConfig getPluginConfig () {
        synchronized (instance) {
            return instance.getPluginInterface ().getPluginconfig ();
        }
    }

    public static final LoggerChannel getLogger () {
        synchronized (instance) {
            return instance.logger;
        }
    }

}
