/*
 * Copyright (C) 2006  Chris Rose
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

package com.aimedia.azjabber;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadListener;
import org.gudy.azureus2.plugins.download.DownloadManager;
import org.gudy.azureus2.plugins.download.DownloadManagerListener;
import org.gudy.azureus2.plugins.utils.LocaleUtilities;

import com.aimedia.azjabber.config.IAzJabberMessageKeys;
import com.aimedia.azjabber.config.IStateTransitionTrackingListener;
import com.aimedia.azjabber.services.ICommunicationServiceProvider;
import com.aimedia.azjabber.services.IServiceConnectionEventListener;
import com.aimedia.azjabber.xmpp.XMPPConnectionManager;

public class NotificationManager implements DownloadManagerListener, DownloadListener,
        IStateTransitionTrackingListener, IServiceConnectionEventListener {

    private static NotificationManager _instance = null;

    public static final NotificationManager instance () {
        if (_instance == null) {
            _instance = new NotificationManager ();
        }
        return _instance;
    }

    private LocaleUtilities              locale;

    private String                       recipientPrefix;

    private Set<String>                  recipientIds;

    private Map<StateTransition, String> transitionMap;

    private Set<ICommunicationServiceProvider> offlineServices;

    private NotificationManager () {
        this.locale = AzJabberPlugin.getPlugin ().getLocale ();

        this.recipientIds = Collections.synchronizedSet (new HashSet<String> ());
        this.transitionMap = Collections.synchronizedMap (new HashMap<StateTransition, String> ());
        this.offlineServices = Collections.synchronizedSet (new HashSet<ICommunicationServiceProvider> ());
    }
    
    public void allServicesConfigured () {
        offlineServices.add (XMPPConnectionManager.instance ());
    }

    public void signalEvent (String eventMessage) {
        if (offlineServices.contains (XMPPConnectionManager.instance ())) {
            if (XMPPConnectionManager.instance ().checkConnection ()) {
                offlineServices.remove (XMPPConnectionManager.instance ());
            }
            Logger.warn (IAzJabberMessageKeys.LOG_MESSAGES_LIKELY_NOT_SENT, eventMessage);
        }
        for (Iterator<String> iter = recipientIds.iterator (); iter.hasNext ();) {
            String resource = iter.next ();
            String to = recipientPrefix + "/" + resource;

            XMPPConnectionManager.instance ().sendMessage (to, eventMessage);
        }
    }

    public void downloadAdded (Download download) {
        if (AzJabberPlugin.getPlugin ().getConfiguration ().isTrackingAddedDownloads ()) {
            signalEvent (locale.getLocalisedMessageText (IAzJabberMessageKeys.TORRENT_ADDED, new String[] { download
                    .getName () }));
        }

        download.addListener (this);
        // downloads.add (download);
    }

    public void downloadRemoved (Download download) {
        download.removeListener (this);
        // downloads.remove (download);

        if (AzJabberPlugin.getPlugin ().getConfiguration ().isTrackingRemovedDownloads ()) {
            signalEvent (locale.getLocalisedMessageText (IAzJabberMessageKeys.TORRENT_REMOVED, new String[] { download
                    .getName () }));
        }
    }

    public void positionChanged (Download download, int oldPosition, int newPosition) {
        // : NO-OP
    }

    public void stateChanged (Download download, int oldState, int newState) {
        StateTransition tx = StateTransition.forStates (oldState, newState);
        if (tx == null) {
            Logger.error (IAzJabberMessageKeys.LOG_INVALID_STATE_TRANSITION, new Object [] {oldState, newState});
        }
        else {
            String message = transitionMap.get (tx);
            if (message != null) {
                signalEvent (locale.getLocalisedMessageText (message, new String[] { download.getName () }));
            }
            else {
                Logger.info (IAzJabberMessageKeys.LOG_DISCARD_STATE_NOTIFICATION, new Object[] {tx, download.getName()});
            }
        }
    }

    public void disconnectFromAllDownloads (DownloadManager mgr) {
        synchronized (mgr) {
            Download[] dlist = mgr.getDownloads ();
            for (int i = 0; i < dlist.length; i++) {
                dlist[i].removeListener (this);
            }
        }
    }

    public void connectToAllDownloads (DownloadManager mgr) {
        synchronized (mgr) {
            Download[] dlist = mgr.getDownloads ();
            for (int i = 0; i < dlist.length; i++) {
                dlist[i].removeListener (this);
                dlist[i].addListener (this);
            }
        }
    }

    public void notifyOnStateTransition (StateTransition transition, String message, boolean notify) {
        if (notify) {
            transitionMap.put (transition, message);
        }
        else {
            transitionMap.remove (transition);
        }
    }

    public void handlingChanged (StateTransition transition, boolean enabled) {
        Logger.info (IAzJabberMessageKeys.LOG_HANDLE_EVENTS, new Object[] {transition.getName (), Boolean.valueOf (enabled)});
        notifyOnStateTransition (transition, IAzJabberMessageKeys.EVENT_MSG_PREFIX + transition.getName ()
                + IAzJabberMessageKeys.EVENT_MSG_SUFFIX, enabled);
    }

    public void recipientOffline (String jid) {
        recipientIds.remove (jid.substring (getRecipientPrefix ().length () + 1));
    }

    public void recipientOnline (String jid) {
        if (jid.startsWith (getRecipientPrefix ())) {
            recipientIds.add (jid.substring (getRecipientPrefix ().length () + 1));
        }
    }

    /**
     * @return the recipient
     */
    public String getRecipientPrefix () {
        return recipientPrefix;
    }

    /**
     * @param recipient
     *            the recipient to set
     */
    public void setRecipientPrefix (String recipient) {
        this.recipientPrefix = recipient;
    }

    public void serviceOffline (ICommunicationServiceProvider provider) {
        recipientIds.clear ();
        
        reconnectService (provider);
    }

    private void reconnectService (ICommunicationServiceProvider provider) {
        offlineServices.add (provider);
    }

}
